// DispStruct.cpp : Implementation of CDispStruct
#include "stdafx.h"
#include "StrucScr.h"
#include "DspStruc.h"
#include "TLibBind.h"

///////////////////////////////////////////////////////////
// CDispStruct


CDispStruct::~CDispStruct()
{
    DestroyRecord();
    CleanNamesArray();
}


ITypeInfoPtr CDispStruct::GetTypeInfo (
    LCID lcid
)
{
    ITypeInfoPtr spTypeInfo;
    ITypeLibPtr spTypeLib;
    HRESULT hr = LoadRegTypeLib(LIBID_StructScript, 1, 0, lcid, 
                 &spTypeLib);
    if (SUCCEEDED(hr)) {
        hr = spTypeLib->GetTypeInfoOfGuid(DIID_IDispStruct, 
             &spTypeInfo);
    }

    return spTypeInfo;
}


// IDispatch methods 

STDMETHODIMP CDispStruct::GetTypeInfoCount (
    UINT* pctinfo
)
{
    if (!pctinfo) return E_POINTER;
    *pctinfo = 0;
    return S_OK;
}

STDMETHODIMP CDispStruct::GetTypeInfo (
    UINT iTInfo, 
    LCID lcid,
    ITypeInfo** ppTInfo
)
{
    if (!ppTInfo) return E_POINTER;
    if (iTInfo != 0) return DISP_E_BADINDEX;
    
    return TYPE_E_ELEMENTNOTFOUND;
}

class BSTRCompare: 
    public std::binary_function<LPOLESTR, LPOLESTR, bool>
{
public:
    bool operator() (LPOLESTR bstr1, LPOLESTR bstr2) const
    {
        int res = ::_wcsicmp(bstr1, bstr2);
        return res == 0;
    }
};

STDMETHODIMP CDispStruct::GetIDsOfNames (
    REFIID riid, 
    LPOLESTR* rgszNames,
    UINT cNames, 
    LCID lcid, 
    DISPID* rgDispId
)
{
    if (riid != IID_NULL) return E_FAIL;
    HRESULT hr = S_OK;

    ITypeInfoPtr spTypeInfo = GetTypeInfo(lcid);
    if (spTypeInfo != 0) {
        hr = ::DispGetIDsOfNames(spTypeInfo, rgszNames, cNames, 
                                 rgDispId);
        if (hr == DISP_E_UNKNOWNNAME) {
            NamesArray_t::iterator pos = 
                std::find_if(m_lstMemberNames.begin(), 
                    m_lstMemberNames.end(), 
                    std::bind1st(BSTRCompare(), rgszNames[0]));

            if (pos != m_lstMemberNames.end()) {
                hr = S_OK;
                rgDispId[0] = 
                (pos - m_lstMemberNames.begin()) + m_dispidMin;
            }
            else {
                hr = DISP_E_UNKNOWNNAME;
                rgDispId[0] = DISPID_UNKNOWN;
            }
        }

        // No named arguments accepted
        for (DISPID id = 1; id < cNames; ++id) {
            rgDispId[id] = DISPID_UNKNOWN;
        }
        if (cNames > 1) hr = DISP_E_UNKNOWNNAME;
    }
    else hr = E_FAIL;
    return hr;
}

STDMETHODIMP CDispStruct::Invoke (DISPID dispIdMember, REFIID riid,
     LCID lcid, WORD wFlags, DISPPARAMS* pDispParams,
     VARIANT* pVarResult, EXCEPINFO* pExcepInfo, UINT* puArgErr)
{
    if (pDispParams->cNamedArgs > 1) return DISP_E_NONAMEDARGS;
    HRESULT hr = S_OK;

    try {
        switch (dispIdMember) {
        case 0: { // Struct property, Default value
            if (wFlags & DISPATCH_PROPERTYGET) {
                if (!pVarResult) 
                    _com_issue_error(E_POINTER);

                *pVarResult = GetRecord();
            }
            else if (wFlags & DISPATCH_PROPERTYPUT) {
                if (pDispParams->cArgs != 1)
                    _com_issue_error(DISP_E_BADPARAMCOUNT);

                SetRecord(pDispParams->rgvarg[0]);
            }
            else 
                _com_issue_error(DISP_E_MEMBERNOTFOUND);
            break;
        }
        case 1: { // CreateRecord method
            if (!(wFlags & DISPATCH_METHOD))
                _com_issue_error(DISP_E_MEMBERNOTFOUND);
            if (pDispParams->cArgs != 2)
                _com_issue_error(DISP_E_BADPARAMCOUNT);
            if ((pDispParams->rgvarg[0].vt != VT_BSTR) || 
                (pDispParams->rgvarg[1].vt != VT_BSTR))
                _com_issue_error(DISP_E_TYPEMISMATCH);

            CreateRecord(pDispParams->rgvarg[1].bstrVal, 
                         pDispParams->rgvarg[0].bstrVal);
            break;
        }
        default: { // Dynamic methods
            if (dispIdMember < m_dispidMin || 
                dispIdMember > m_dispidMax)
                _com_issue_error(DISP_E_MEMBERNOTFOUND);
            BSTR bstrName = 
                m_lstMemberNames[dispIdMember - (m_dispidMin)];
            if (wFlags & DISPATCH_PROPERTYGET) {
                if (!pVarResult) _com_issue_error(E_POINTER);
                *pVarResult = GetField(dispIdMember, bstrName);
            }
            else if (wFlags & DISPATCH_PROPERTYPUT) {
                if (pDispParams->cArgs != 1)
                    _com_issue_error(DISP_E_BADPARAMCOUNT);
                PutField(bstrName, *pDispParams->rgvarg);
            }
            else 
                _com_issue_error(DISP_E_MEMBERNOTFOUND);
            break;
        }
        }
    }
    catch (_com_error& e) {
        hr = e.Error();
    }

    return hr;
}


void CDispStruct::PutField (
    BSTR bstrName, 
    VARIANT& vaField
)
{
    HRESULT hr = S_OK;
    unsigned long lFlags = 0x4;
    if (vaField.vt == VT_DISPATCH) {
        // Check if the IDispatch pointer corresponds to a 
        // DispStruct object
        IDispatch* pdispContent = 0;
        hr = vaField.pdispVal->QueryInterface(DIID_IDispStruct, 
                    reinterpret_cast<void**>(&pdispContent));
        if (SUCCEEDED(hr)) {
            VARIANT vaRealField;
            hr = ::_com_dispatch_propget(pdispContent, 0, 
                                VT_VARIANT, &vaRealField);
            ::VariantClear(&vaField);
            vaField = vaRealField;
            pdispContent->Release();
        }
    }

    hr = m_spRecordInfo->PutField(lFlags, m_pRecord, 
                            bstrName, &vaField);
}


VARIANT CDispStruct::GetField (
    DISPID dispIdMember, 
    BSTR bstrName
)
{
    HRESULT hr = S_OK;
    _variant_t vaField;

    // Need to know first what type is the field.
    // Records must be referenced, not copied
    ITypeInfoPtr spTypeInfo;
    VARDESC* pvardesc;
    hr = m_spRecordInfo->GetTypeInfo(&spTypeInfo);
    hr = spTypeInfo->GetVarDesc((dispIdMember - m_dispidMin), 
                                   &pvardesc);
    
    if (pvardesc->elemdescVar.tdesc.vt != VT_USERDEFINED) {
        hr = m_spRecordInfo->GetField(m_pRecord, bstrName, 
                                         &vaField);
        if (FAILED(hr)) _com_issue_error(hr);
    }
    else {
        // If the field value is a record, 
        // convert it to a dispatch value
        CComObject<CDispStruct>* pdispStruct = 0;
        CComObject<CDispStruct>::CreateInstance(&pdispStruct);
        pdispStruct->AddRef();

        pdispStruct->LinkToOuterRecord(this, spTypeInfo, pvardesc);
        // Do not VariantClear vaRecord. 
        // The contents have been attached to the object 
        // pointed to by this instance without AddRefing 
        // or copying so the clear operation would destroy 
        // the actual data that will be returned
        vaField.vt = VT_DISPATCH;
        vaField.pdispVal = static_cast<IDispatch*>(pdispStruct);
    }

    spTypeInfo->ReleaseVarDesc(pvardesc);
    return vaField.Detach();
}


void CDispStruct::CreateRecord(BSTR TypeLib, BSTR TypeName)
{
    ITypeLibPtr spTypeLib;
    CTypeLibBinder tlBinder;

    HRESULT hr = tlBinder.BindToTypeLib(TypeLib, &spTypeLib);
    if (FAILED(hr)) _com_issue_error(hr);

    ITypeInfoPtr spTypeInfo;
    MEMBERID nMemberID = 0;
    unsigned short nCount = 1;
    hr = spTypeLib->FindName(TypeName, 0, &spTypeInfo, 
                             &nMemberID, &nCount);

    if (FAILED(hr) || (spTypeInfo == 0)) 
        _com_issue_error(E_INVALIDARG);

    DestroyRecord();
    hr = ::GetRecordInfoFromTypeInfo(spTypeInfo, &m_spRecordInfo);
    if (FAILED(hr)) _com_issue_error(hr);
    m_pRecord = m_spRecordInfo->RecordCreate();
    m_spRecordInfo->RecordInit(m_pRecord);
    SetupAutomationStruct();
}


VARIANT CDispStruct::GetRecord (
)
{
    VARIANT vaResult;
    ::VariantInit(&vaResult);

    if (m_pRecord != 0) {
        vaResult.vt = VT_RECORD;
        vaResult.pRecInfo = m_spRecordInfo;
        vaResult.pRecInfo->AddRef();
        m_spRecordInfo->RecordCreateCopy(m_pRecord, 
                               &vaResult.pvRecord);
    }
    else {
        vaResult.vt = VT_NULL;
    }

    return vaResult;
}


void CDispStruct::SetRecord (
    const VARIANT& vaRecord
)
{
    if (vaRecord.vt != VT_RECORD) 
        _com_issue_error(DISP_E_TYPEMISMATCH);

    DestroyRecord();
    m_spRecordInfo = vaRecord.pRecInfo;
    m_spRecordInfo->RecordCreateCopy(vaRecord.pvRecord, 
                                     &m_pRecord);
    SetupAutomationStruct();
}


void CDispStruct::LinkToOuterRecord (
    CDispStruct* pOuterRecord, 
    ITypeInfoPtr spTypeInfo, 
    VARDESC* pvardesc
)
{
    DestroyRecord();

    // Try to get a IRecordInfo instance refered to the nested type
    ITypeInfoPtr spNestedTypeInfo;
    HRESULT hr = 
        spTypeInfo->GetRefTypeInfo(
                    pvardesc->elemdescVar.tdesc.hreftype, 
                    &spNestedTypeInfo);
    hr = ::GetRecordInfoFromTypeInfo(spNestedTypeInfo,
                                     &m_spRecordInfo);

    // Attach the record to the actual instance 
    // in the outer record
    m_pRecord = reinterpret_cast<BYTE*>(pOuterRecord->m_pRecord) 
                + pvardesc->oInst;

    // Keep a strong reference to the outer record object
    // This way, if the outer object goes out of scope, 
    // the data remains valid due to this reference
    m_pOuterRecord = pOuterRecord;
    m_pOuterRecord->AddRef();

    SetupAutomationStruct();
}


void CDispStruct::DestroyRecord (
)
{
    if (m_pOuterRecord) {
        m_pOuterRecord->Release();
        m_pOuterRecord = 0;
    }
    else if (m_pRecord) {
        // m_pRecord is owned by this instance if and only if 
        // m_pOuterRecord is NULL.
        // Otherwise, m_pRecord is a reference to the record 
        // owned by the OuterRecord
        m_spRecordInfo->RecordDestroy(m_pRecord);
        m_pRecord = 0;
    }
    // Independently, the RecordInfo reference must be released
    m_spRecordInfo = 0;
}

void CDispStruct::CleanNamesArray (
)
{
    for (NamesArray_t::iterator pos = m_lstMemberNames.begin(); 
         pos != m_lstMemberNames.end(); ++pos) {
        ::SysFreeString(*pos);
        *pos = 0;
    }
}

void CDispStruct::SetupAutomationStruct (
)
{
    if (m_spRecordInfo == 0) return;
    HRESULT hr = S_OK;

    ITypeInfoPtr spTypeInfo;
    TYPEATTR* ptattr = 0;
    hr = m_spRecordInfo->GetTypeInfo(&spTypeInfo);
    hr = spTypeInfo->GetTypeAttr(&ptattr);

    // DISPID-Member name relationship must remain constant 
    // through the life of the object.
    // So, each time the record changes, the dispid map must be 
    // recreated
    unsigned long nMembers = ptattr->cVars;
    CleanNamesArray();
    m_dispidMin = ((m_dispidMax + 10) / 10 ) + 1;
    m_dispidMax = m_dispidMin + nMembers - 1;

    m_lstMemberNames.resize(nMembers);
    m_spRecordInfo->GetFieldNames(&nMembers, 
                    m_lstMemberNames.begin());

    spTypeInfo->ReleaseTypeAttr(ptattr);
}

