// Message.cpp : Implementation of WinMain
#include "stdafx.h"
#include "Resrc.h"
#include <initguid.h>
#include "Message.h"
#include "MsgDlg.h"
#include "Message_i.c"
#include "SimplMsg.h"

const DWORD dwTimeOut = 5000;     // idle time before shutting down
const DWORD dwPause = 1000;   // time to wait for threads to finish

CMessageDialog* CMessageDialog::psThis;

static DWORD WINAPI MonitorProc(void* pv) // Passed to CreateThread 
{                                  // to monitor the shutdown event
    CExeModule* p = (CExeModule*)pv;
    p->MonitorShutdown();
    return 0;
}

LONG CExeModule::Unlock()
{
    LONG l = CComModule::Unlock();
    if (l == 0)
    {
        bActivity = true;
        SetEvent(hEventShutdown);                   // tell monitor
    }
    return l;
}

void CExeModule::MonitorShutdown()    //Monitors the shutdown event
{
    while (1)
    {
        WaitForSingleObject(hEventShutdown, INFINITE);
        DWORD dwWait=0;
        do
        {
            bActivity = false;
            dwWait = WaitForSingleObject(hEventShutdown, dwTimeOut);
        } while (dwWait == WAIT_OBJECT_0);
        // timed out
        if (!bActivity && m_nLockCnt == 0)   // exit if no activity
        {
            break;
        }
    }
    CloseHandle(hEventShutdown);
    PostThreadMessage(dwThreadID, WM_QUIT, 0, 0);
}

bool CExeModule::StartMonitor()
{
    hEventShutdown = CreateEvent(NULL, false, false, NULL);
    if (hEventShutdown == NULL) return false;
    DWORD dwThreadID;
    HANDLE h = CreateThread(NULL, 0, MonitorProc, this, 0, 
	               &dwThreadID);
    return (h != NULL);
}

__declspec(dllexport) CExeModule _Module;

BEGIN_OBJECT_MAP(ObjectMap)
OBJECT_ENTRY(CLSID_SimpleMessage, CSimpleMessage)
END_OBJECT_MAP()

LPCTSTR FindOneOf(LPCTSTR p1, LPCTSTR p2)
{
    while (p1 != NULL && *p1 != NULL)
    {
        LPCTSTR p = p2;
        while (p != NULL && *p != NULL)
        {
            if (*p1 == *p)
                return CharNext(p1);
            p = CharNext(p);
        }
        p1 = CharNext(p1);
    }
    return NULL;
}

extern "C" int WINAPI _tWinMain(HINSTANCE hInstance, 
    HINSTANCE /*hPrevInstance*/, LPTSTR lpCmdLine, int nShowCmd)
{
    lpCmdLine = GetCommandLine();     // necessary for _ATL_MIN_CRT

    HRESULT hRes = CoInitialize(NULL);
    _ASSERTE(SUCCEEDED(hRes));
    _Module.Init(ObjectMap, hInstance, &LIBID_MESSAGELib);
    _Module.dwThreadID = GetCurrentThreadId();
    TCHAR szTokens[] = _T("-/");

    _Module.SetShowCmd(nShowCmd);

    int nRet = 0;
    BOOL bRun = FALSE;
    BOOL bDisplay = TRUE;
    LPCTSTR lpszToken = FindOneOf(lpCmdLine, szTokens);
    while (lpszToken != NULL)
    {
        if (lstrcmpi(lpszToken, _T("UnregServer"))==0)
        {
            _Module.UpdateRegistryFromResource(IDR_Message, FALSE);
            nRet = _Module.UnregisterServer(TRUE);
            bRun = FALSE;
            bDisplay = FALSE;
            break;
        }
        if (lstrcmpi(lpszToken, _T("RegServer"))==0)
        {
            _Module.UpdateRegistryFromResource(IDR_Message, TRUE);
            nRet = _Module.RegisterServer(TRUE);
            bRun = FALSE;
            bDisplay = FALSE;
            break;
        }
        if (lstrcmpi(lpszToken, _T("Embedding"))==0)
        {
            bRun = TRUE;
            bDisplay = FALSE;
            break;
        }
        lpszToken = FindOneOf(lpszToken, szTokens);
    }

    if (bRun)
    {
        _Module.StartMonitor();
        hRes = _Module.RegisterClassObjects(CLSCTX_LOCAL_SERVER, 
            REGCLS_SINGLEUSE);
        _ASSERTE(SUCCEEDED(hRes));
        _Module.Lock();

        MSG msg;
        while (GetMessage(&msg, 0, 0, 0))
        {
            HWND hw = CMessageDialog::GetWindow();
            if (!IsWindow(hw) || !IsDialogMessage(hw, &msg)) 
            { 
                TranslateMessage(&msg); 
                DispatchMessage(&msg); 
            } 
        }

        _Module.RevokeClassObjects();
        Sleep(dwPause); //wait for any threads to finish
    }
    if(bDisplay)
        CMessageDialog dialog(true);
    _Module.Term();
    CoUninitialize();
    return nRet;
}
