#define UNICODE
#define _UNICODE

#include <windows.h>
#include <sfc.h>
#include "sysdir.h"

int CALLBACK MainDlgProc(HWND, UINT, WPARAM, LPARAM);
void GetProtectedFiles(HWND hWnd, HWND hListBox);
void ReplaceProtectedFile(LPTSTR szProtectedFile);

HANDLE hInst;
TCHAR sz[MAX_PATH];

//-----------------------------------------------------------------
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR lpCmdLine, int nCmdShow)
{
    hInst = hInstance;
    DialogBoxParam(hInst, MAKEINTRESOURCE(DLG_MAIN), NULL, 
                   MainDlgProc, 0);
    return 0;

} // WinMain

//-----------------------------------------------------------------
int CALLBACK MainDlgProc(HWND hDlg, UINT Message, WPARAM wParam,
                        LPARAM lParam)
{
    HWND hListBox;
    int index;

    switch (Message) {

        case WM_INITDIALOG:
            GetProtectedFiles(hDlg, GetDlgItem(hDlg, 
                              ID_PROTECTED_FILES));
            return TRUE;

        case WM_COMMAND:
            switch (LOWORD(wParam)) {
                case IDCANCEL:
                    EndDialog(hDlg, TRUE);
                    return TRUE;

                case ID_REFRESH:
                    GetProtectedFiles(hDlg, GetDlgItem(hDlg, 
                                      ID_PROTECTED_FILES));
                    break;

                case ID_REPLACE:
                    hListBox = GetDlgItem(hDlg,ID_PROTECTED_FILES);
                    index = SendMessage(hListBox, LB_GETCURSEL,0,0);
					if (index != LB_ERR) {
                        SendMessage(hListBox, LB_GETTEXT, index, 
                                    (WPARAM)sz);
                        ReplaceProtectedFile(sz);
                    } else {
                        MessageBeep(0);
                    } 
                    break;

                case ID_CHECK_FILE:
                    if (GetDlgItemText(hDlg, ID_FILENAME, sz, 
                                       MAX_PATH)) {
                        if (SfcIsFileProtected(NULL, sz)) {
                            SetDlgItemText(hDlg, ID_FILE_STATUS,
                                           SZ_PROTECTED);
                        } else {
                            SetDlgItemText(hDlg, ID_FILE_STATUS,
                                           SZ_UNPROTECTED);
                        }
                    }
                    break;
            }

        default:
            return FALSE;
    }

} // MainDlgProc

//-----------------------------------------------------------------
void GetProtectedFiles(HWND hWnd, HWND hListBox)
{
    PROTECTED_FILE_DATA pfd;

    SendMessage(hListBox, LB_RESETCONTENT, 0, 0);
    pfd.FileNumber = 0;

    while (SfcGetNextProtectedFile(NULL, &pfd)) {
        SendMessage(hListBox, LB_ADDSTRING,0,(LPARAM)pfd.FileName);
    } 

    return;

} // GetProtectedFiles

//-----------------------------------------------------------------
void ReplaceProtectedFile(LPTSTR szProtectedFile)
{
    // Attempt to copy a temp file over the specified protected
    // file - this should cause a warning message and the cached
    // version of the original file should get copied back to the
    // working location.

    if (!CopyFile(SZ_TEMPFILE, szProtectedFile, FALSE)) {
        wsprintf(sz, L"CopyFile failed (%x)", GetLastError());
        MessageBox(NULL, sz, TEXT("SysDir"), MB_OK);
    }

    return;

} // ReplaceProtectedFile
