// TLibBind.cpp : Implementation of CTypeLibBinder

#include "stdafx.h"
#include "TLibBind.h"



HRESULT CTypeLibBinder::BindToTypeLib (
    LPOLESTR pszDisplayName, 
    ITypeLib** ppTypeLib
)
{
    if (!ppTypeLib) return E_POINTER;
    *ppTypeLib = 0;
    HRESULT hr = S_OK;
    try {
        if (IsGUID(pszDisplayName)) {
            *ppTypeLib = LoadFromGUID(pszDisplayName);
        }
        else {
            *ppTypeLib = LoadFromFile(pszDisplayName);
        }
    }
    catch (...) {
        hr = E_FAIL;
    }

    return hr;
}


bool CTypeLibBinder::IsGUID (
    LPOLESTR pszDisplayName
)
{
    return (pszDisplayName[0] == '{');
}


ITypeLib* CTypeLibBinder::LoadFromGUID (
    LPOLESTR pszGUID
)
{
    ITypeLib* pTypeLib = 0;
    GUID pguid;
    WORD wVerMajor = 1;
    WORD wVerMinor = 0;


    // Check whether the given GUID includes 
    // the type library version information,
    // use default version value 1.0 as default otherwise
    LPOLESTR pszVersionInfo = ::wcschr(pszGUID, '!');
    if (pszVersionInfo) {
        wVerMajor = ::_wtoi(++pszVersionInfo);
        LPOLESTR pszVersionMinor = ::wcschr(pszVersionInfo, '.');
        if (pszVersionMinor) {
            wVerMinor = ::_wtoi(++pszVersionMinor);
        }
    }

    HRESULT hr = ::IIDFromString(pszGUID, &pguid);
    if (FAILED(hr)) throw hr;

    hr = ::LoadRegTypeLib(pguid, wVerMajor, wVerMinor, 
            LOCALE_SYSTEM_DEFAULT, &pTypeLib);
    if (FAILED(hr)) throw hr;

    return pTypeLib;
}


ITypeLib* CTypeLibBinder::LoadFromFile (
    LPOLESTR pszFileName
)
{
    ITypeLib* pTypeLib = 0;
    HRESULT hr = ::LoadTypeLib(pszFileName, &pTypeLib);
    if (FAILED(hr)) throw hr;

    return pTypeLib;
}

